!  DLLDIP8CUBEN.f90 
!
subroutine DLLDIP8CUBEN (lvector,gtensor,rvalue,brealmatrixa,bimagmatrixa,brealmatrixb,bimagmatrixb,zerorealmatrix,zeroimagmatrix)

  ! DLLDip8CubeN fills up the Energy Matrix for a system with a central S=1/2 spins subject to dipolar interaction with 8 indentical S = 1/2 neighbors at the corners of a cube.
  ! NOTE: This subroutine has a lot of overkill; it is ready to take tho spins with different g-tensors whereby the second tensor may be rotated with respect to the first;
  !       it can also take 14 neighbors in a BCC lattice. All these options are 'commented out'.
  !
  ! Input is:   the lvector (lx, ly, lz; or: cos(theta), sin(theta)sin(phi), sin(theta)cos(phi)) between B and the g-tensor of spin-A
  !             the diagonal g-tensor of spin-A and its neighbors
  !             the rdistance (rvalue) calculated in the MAIN program from a stochastic distribution
  !
  ! Output is:  the Energy matrix (in units of reciprocal cm) split up into tree partial matrices, namely
  !             brealmatrix is the real part of the Zeeman interaction for B = 1
  !             bimagmatrix are the coefficients of the imaginary part of the Zeeman interaction (i.e. not yet multiplied by i) for B = 1
  !             zerorealmatrix and zeroimagmatrix contain the zero-field interactions from exchange and dipole-dipole between the two spins
  !
  ! History:    DLLDipolar6         20191028
  !             DLLDipolar14        20191207
  !             DLLDIP14ASYM        20191218
  !             DLLDIP14STOCHASTICN 20201215
  !
  ! (C) WRHagen 2021 01 20

    IMPLICIT NONE
  !DEC$ ATTRIBUTES DLLEXPORT::DLLDIP8CUBEN
  !DEC$ ATTRIBUTES STDCALL::DLLDIP8CUBEN

  ! Declare calling parameters 
    REAL*8, INTENT(IN) ::  lvector(3), gtensor(3,3), rvalue
    REAl*8, INTENT(OUT) :: brealmatrixa(4,4), bimagmatrixa(4,4), brealmatrixb(4,4), bimagmatrixb(4,4), zerorealmatrix(4,4), zeroimagmatrix(4,4)

  ! Declare local parameters
    REAL*8  rvector(3), rdistance
    
    REAL*8  Svectora11(3), Svectora13(3), Svectora22(3), Svectora24(3)
    REAL*8  Svectora31(3), Svectora33(3), Svectora42(3), Svectora44(3)
    REAL*8  Cvectora13(3), Cvectora24(3), Cvectora31(3), Cvectora42(3)
    
    REAL*8  Svectorb11(3), Svectorb12(3), Svectorb21(3), Svectorb22(3)
    REAL*8  Svectorb33(3), Svectorb34(3), Svectorb43(3), Svectorb44(3)
    REAL*8  Cvectorb12(3), Cvectorb21(3), Cvectorb34(3), Cvectorb43(3)
    
    REAL*8  beta, SIfactor, dummy
    REAL*8  vectorlga(3), vectorlgb(3)
    
    INTEGER i, j
    INTEGER k
    
    REAL*8  drealmatrix1(4,4), dimagmatrix1(4,4)
    REAL*8  ax, ay, az, bx, by, bz  ! Dummy elements for d####matrix2
    REAL*8  drealmatrix2(4,4), dimagmatrix2(4,4)
    
  ! Body of DLLDipolar
    
! Svectors contain real elements; Cvectors contain imaginary elements: 
    
    Svectora11(1) = 0.0; Svectora11(2) = 0.0; Svectora11(3) = 0.5
    Svectora13(1) = 0.5; Svectora13(2) = 0.0; Svectora13(3) = 0.0
    Cvectora13(1) = 0.0; Cvectora13(2) =-0.5; Cvectora13(3) = 0.0
    Svectora22(1) = 0.0; Svectora22(2) = 0.0; Svectora22(3) = 0.5
    Svectora24(1) = 0.5; Svectora24(2) = 0.0; Svectora24(3) = 0.0
    Cvectora24(1) = 0.0; Cvectora24(2) =-0.5; Cvectora24(3) = 0.0
    Svectora31(1) = 0.5; Svectora31(2) = 0.0; Svectora31(3) = 0.0
    Cvectora31(1) = 0.0; Cvectora31(2) = 0.5; Cvectora31(3) = 0.0
    Svectora33(1) = 0.0; Svectora33(2) = 0.0; Svectora33(3) =-0.5
    Svectora42(1) = 0.5; Svectora42(2) = 0.0; Svectora42(3) = 0.0
    Cvectora42(1) = 0.0; Cvectora42(2) = 0.5; Cvectora42(3) = 0.0
    Svectora44(1) = 0.0; Svectora44(2) = 0.0; Svectora44(3) =-0.5
    
    Svectorb11(1) = 0.0; Svectorb11(2) = 0.0; Svectorb11(3) = 0.5
    Svectorb12(1) = 0.5; Svectorb12(2) = 0.0; Svectorb12(3) = 0.0
    Cvectorb12(1) = 0.0; Cvectorb12(2) =-0.5; Cvectorb12(3) = 0.0
    Svectorb21(1) = 0.5; Svectorb21(2) = 0.0; Svectorb21(3) = 0.0
    Cvectorb21(1) = 0.0; Cvectorb21(2) = 0.5; Cvectorb21(3) = 0.0
    Svectorb22(1) = 0.0; Svectorb22(2) = 0.0; Svectorb22(3) =-0.5
    Svectorb33(1) = 0.0; Svectorb33(2) = 0.0; Svectorb33(3) = 0.5
    Svectorb34(1) = 0.5; Svectorb34(2) = 0.0; Svectorb34(3) = 0.0
    Cvectorb34(1) = 0.0; Cvectorb34(2) =-0.5; Cvectorb34(3) = 0.0 
    Svectorb43(1) = 0.5; Svectorb43(2) = 0.0; Svectorb43(3) = 0.0
    Cvectorb43(1) = 0.0; Cvectorb43(2) = 0.5; Cvectorb43(3) = 0.0
    Svectorb44(1) = 0.0; Svectorb44(2) = 0.0; Svectorb44(3) =-0.5
    
    beta = 0.46689088 / 10000.  ! in reciprocal cm per gauss
    
    vectorlga = beta * matmul(lvector,gtensor)
    vectorlgb = vectorlga    ! A and B are identical molecules with co-linear g-tensors
    
! Initialize all partial energy matrices:
    
    FORALL (i=1:4,j=1:4)
        brealmatrixa(i,j) = 0.0
        bimagmatrixa(i,j) = 0.0
        brealmatrixb(i,j) = 0.0
        bimagmatrixb(i,j) = 0.0

        drealmatrix1(i,j) = 0.0
        dimagmatrix1(i,j) = 0.0
        drealmatrix2(i,j) = 0.0
        dimagmatrix2(i,J) = 0.0
    END FORALL
    
! Zeeman terms for spin-A and spin-B:
    
    brealmatrixa(1,1) = dot_product(vectorlga, Svectora11)
    brealmatrixa(1,3) = dot_product(vectorlga, Svectora13)
    brealmatrixa(2,2) = dot_product(vectorlga, Svectora22)
    brealmatrixa(2,4) = dot_product(vectorlga, Svectora24)
    brealmatrixa(3,1) = dot_product(vectorlga, Svectora31)
    brealmatrixa(3,3) = dot_product(vectorlga, Svectora33)
    brealmatrixa(4,2) = dot_product(vectorlga, Svectora42)
    brealmatrixa(4,4) = dot_product(vectorlga, Svectora44)

    bimagmatrixa(1,3) = dot_product(vectorlga, Cvectora13)
    bimagmatrixa(2,4) = dot_product(vectorlga, Cvectora24)
    bimagmatrixa(3,1) = dot_product(vectorlga, Cvectora31)
    bimagmatrixa(4,2) = dot_product(vectorlga, Cvectora42)
    
    brealmatrixb(1,1) = dot_product(vectorlgb, Svectorb11)
    brealmatrixb(1,2) = dot_product(vectorlgb, Svectorb12)
    brealmatrixb(2,1) = dot_product(vectorlgb, Svectorb21)
    brealmatrixb(2,2) = dot_product(vectorlgb, Svectorb22)
    brealmatrixb(3,3) = dot_product(vectorlgb, Svectorb33)
    brealmatrixb(3,4) = dot_product(vectorlgb, Svectorb34)
    brealmatrixb(4,3) = dot_product(vectorlgb, Svectorb43)   
    brealmatrixb(4,4) = dot_product(vectorlgb, Svectorb44)       

    bimagmatrixb(1,2) = dot_product(vectorlgb, Cvectorb12)    
    bimagmatrixb(2,1) = dot_product(vectorlgb, Cvectorb21)    
    bimagmatrixb(3,4) = dot_product(vectorlgb, Cvectorb34)
    bimagmatrixb(4,3) = dot_product(vectorlgb, Cvectorb43)    

! First term of dipolar interaction (non-colinear terms are commented out):

    drealmatrix1(1,4) = gtensor(1,1)*gtensor(1,1) - gtensor(2,2)*gtensor(2,2)
!    dimagmatrix1(1,4) = -gtensor(1,1)*gtensor(1,2) - gtensor(2,2)*gtensor(2,1)
    drealmatrix1(2,3) = gtensor(1,1)*gtensor(1,1) + gtensor(2,2)*gtensor(2,2)
!    dimagmatrix1(2,3) = gtensor(1,1)*gtensor(1,2) - gtensor(2,2)*gtensor(2,1)
    drealmatrix1(3,2) = gtensor(1,1)*gtensor(1,1) + gtensor(2,2)*gtensor(2,2)
!    dimagmatrix1(3,2) = -gtensor(1,1)*gtensor(1,2) + gtensor(2,2)*gtensor(2,1) 
    drealmatrix1(4,1) = gtensor(1,1)*gtensor(1,1) - gtensor(2,2)*gtensor(2,2)
!    dimagmatrix1(4,1) = gtensor(1,1)*gtensor(1,2) + gtensor(2,2)*gtensor(2,1)
!    drealmatrix1(1,3) = gtensor(1,1)*gtensor(1,3)
!    dimagmatrix1(1,3) = -gtensor(2,2)*gtensor(2,3)
!    drealmatrix1(3,1) = drealmatrix1(1,3)
!    dimagmatrix1(3,1) = dimagmatrix1(1,3)
!    drealmatrix1(2,4) = drealmatrix1(1,3)
!    dimagmatrix1(2,4) = -dimagmatrix1(1,3)
!    drealmatrix1(4,2) = drealmatrix1(3,1)
!    dimagmatrix1(4,2) = dimagmatrix1(3,1)
!    drealmatrix1(1,2) = gtensor(3,3)*gtensor(3,1)
!    dimagmatrix1(1,2) = -gtensor(3,3)*gtensor(3,2)
!    drealmatrix1(2,1) = drealmatrix1(1,2)
!    dimagmatrix1(2,1) = dimagmatrix1(1,2)
!    drealmatrix1(3,4) = drealmatrix1(1,2)
!    dimagmatrix1(3,4) = -dimagmatrix1(1,2)
!    drealmatrix1(4,3) = drealmatrix1(2,1)
!    dimagmatrix1(4,3) = dimagmatrix1(2,1)
    drealmatrix1(1,1) = gtensor(3,3)*gtensor(3,3)
    drealmatrix1(4,4) = drealmatrix1(1,1)
    drealmatrix1(2,2) = -drealmatrix1(1,1)
    drealmatrix1(3,3) = drealmatrix1(2,2)

    drealmatrix1 = drealmatrix1 / 4.    ! (book 13, page 55)
    dimagmatrix1 = dimagmatrix1 / 4.
    
! Second term of dipolar interaction:
    
    DO  k=1,8       ! cubic surrounding (BCC next nearest neighbors are commented out)
        SELECT CASE(k)
        CASE(1)
            rvector(1)=0.5; rvector(2)=0.5; rvector(3)=0.5
        CASE(2)
            rvector(1)=0.5; rvector(2)=0.5; rvector(3)=-0.5
        CASE(3)
            rvector(1)=-0.5; rvector(2)=0.5; rvector(3)=-0.5
        CASE(4)
            rvector(1)=-0.5; rvector(2)=0.5; rvector(3)=0.5
        CASE(5)
            rvector(1)=0.5; rvector(2)=-0.5; rvector(3)=0.5    
        CASE(6)
            rvector(1)=0.5; rvector(2)=-0.5; rvector(3)=-0.5     
        CASE(7)
            rvector(1)=-0.5; rvector(2)=-0.5; rvector(3)=-0.5    
        CASE(8)
            rvector(1)=-0.5; rvector(2)=-0.5; rvector(3)=0.5       
!        CASE(9)
!            rvector(1)=1.; rvector(2)=0.; rvector(3)=0.
!        CASE(10)
!            rvector(1)=0.; rvector(2)=1.; rvector(3)=0.
!        CASE(11)
!            rvector(1)=-1.; rvector(2)=0.; rvector(3)=0.
!        CASE(12)
!            rvector(1)=0.; rvector(2)=-1.; rvector(3)=0.
!        CASE(13)
!            rvector(1)=0.; rvector(2)=0.; rvector(3)=1.
!        CASE(14)
!            rvector(1)=0.; rvector(2)=0.; rvector(3)=-1
        CASE DEFAULT
            rvector(1)=999.; rvector(2)=999.; rvector(3)=999.
        END SELECT
        
  !     The nearest neighbor in a cubic corner lattic is at SQRT(3/4)=0.866025 unit lenghts        
        rdistance = rvalue / 0.866025
        rvector = rvector * rdistance
        
    ax = rvector(1)*gtensor(1,1)
    ay = rvector(2)*gtensor(2,2)
    az = rvector(3)*gtensor(3,3)
    bx = rvector(1)*gtensor(1,1) + rvector(2)*gtensor(2,1) + rvector(3)*gtensor(3,1)
    by = rvector(1)*gtensor(1,2) + rvector(2)*gtensor(2,2) + rvector(3)*gtensor(3,2)
    bz = rvector(1)*gtensor(1,3) + rvector(2)*gtensor(2,3) + rvector(3)*gtensor(3,3)
    
    drealmatrix2(1,4) = ax*bx - ay*by
    dimagmatrix2(1,4) = -ax*by - ay*bx
    drealmatrix2(2,3) = ax*by + ay*by
    dimagmatrix2(2,3) = ax*by -ay*bx
    
    drealmatrix2(3,2) = ax*bx + ay*by
    dimagmatrix2(3,2) = -ax*by + ay*bx 
    drealmatrix2(4,1) = ax*bx - ay*by
    dimagmatrix2(4,1) = ax*by + ay*bx
    drealmatrix2(1,3) = ax*bz
    dimagmatrix2(1,3) = -ay*bz
    drealmatrix2(3,1) = drealmatrix2(1,3)
    dimagmatrix2(3,1) = dimagmatrix2(1,3)
    drealmatrix2(2,4) = drealmatrix2(1,3)
    dimagmatrix2(2,4) = -dimagmatrix2(1,3)
    drealmatrix2(4,2) = drealmatrix2(3,1)
    dimagmatrix2(4,2) = dimagmatrix2(3,1)
    drealmatrix2(1,2) = az*bx
    dimagmatrix2(1,2) = -az*by
    drealmatrix2(2,1) = drealmatrix2(1,2)
    dimagmatrix2(2,1) = dimagmatrix2(1,2)
    drealmatrix2(3,4) = drealmatrix2(1,2)
    dimagmatrix2(3,4) = -dimagmatrix2(1,2)
    drealmatrix2(4,3) = drealmatrix2(2,1)
    dimagmatrix2(4,3) = dimagmatrix2(2,1)
    drealmatrix2(1,1) = az*bz
    drealmatrix2(4,4) = drealmatrix2(1,1)
    drealmatrix2(2,2) = -drealmatrix2(1,1)
    drealmatrix2(3,3) = drealmatrix2(2,2)
    drealmatrix2 = -3.0*drealmatrix2 / (rdistance*rdistance)
    dimagmatrix2 = -3.0*dimagmatrix2 / (rdistance*rdistance)
    drealmatrix2 = drealmatrix2 / 4.    ! (book 13, page 57)
    dimagmatrix2 = dimagmatrix2 / 4.

  ! Note that in the point-dipole equation (book 13, page 47):
  ! the first beta is in wavenumbers/gauss since hv is calculated in wavenumbers;
  ! and the second beta is in SI units: joule/tesla.
  ! The pre-factor of the dipolar term is mu-zero x beta-squared / 4 pi x r^3,
  ! wherin beta-squared is first beta (wavenumbers/gauss) times second beta (joule/tesla).
  ! The pre-factor is first-beta x SIfactor.
    
  ! SIfactor is (mu-nul/4pi) * (beta/r**3) with ALL quantities in SI units
  ! and for F in Hertz (not MHz) and rdistance from cm (not m) to Angstrom 
    SIfactor = 9.274009994 * 1000.   
    zerorealmatrix = zerorealmatrix + beta * SIfactor * (drealmatrix1 + drealmatrix2) / (rdistance*rdistance*rdistance)
    zeroimagmatrix = zeroimagmatrix + beta * SIfactor * (dimagmatrix1 + dimagmatrix2) / (rdistance*rdistance*rdistance)
   
    END DO
    
    RETURN

end subroutine DLLDIP8CUBEN
